import _typeof from "@babel/runtime/helpers/esm/typeof";
import _objectSpread from "@babel/runtime/helpers/esm/objectSpread2";
import * as path from 'path';
import { lazyImport, PLUGIN_SCHEMAS, createRuntimeExportsUtils } from '@modern-js/utils';
export default (function () {
  return {
    name: '@modern-js/plugin-design-token',
    setup: function setup(api) {
      var pluginsExportsUtils;
      var designTokenModulePath = path.resolve(__dirname, '../../../../');
      var resolveConfig = lazyImport('tailwindcss/resolveConfig', require);
      var PLUGIN_IDENTIFIER = 'designToken';

      var getDesignTokens = function getDesignTokens(userConfig) {
        var _ref = userConfig,
            designSystem = _ref.source.designSystem; // TODO: Type to be filled

        var tailwindcssConfig = {};
        tailwindcssConfig.theme = designSystem ? _objectSpread({}, designSystem) : {}; // not use default design token when designToken.defaultTheme is false or theme is false

        if (!designSystem) {
          tailwindcssConfig.presets = [];
        } // when only designSystem exist, need remove supportStyledComponents


        if (designSystem) {
          delete tailwindcssConfig.theme.supportStyledComponents;
        }

        return resolveConfig(tailwindcssConfig).theme || {};
      };

      return {
        config: function config() {
          var appContext = api.useAppContext();
          pluginsExportsUtils = createRuntimeExportsUtils(appContext.internalDirectory, 'plugins');
          return {
            source: {
              alias: {
                '@modern-js/runtime/plugins': pluginsExportsUtils.getPath()
              }
            },
            tools: {// TODO: support less、scss、css vars
              // less: https://github.com/modern-js-dev/modern.js/pull/398/files#diff-f77c749e403fbf1fb676d5687bef3d7138a230331c849298d8afaff9b6afbc3dR166
              // sass: https://github.com/modern-js-dev/modern.js/pull/398/files#diff-f77c749e403fbf1fb676d5687bef3d7138a230331c849298d8afaff9b6afbc3dR229
              // postcss: https://github.com/modern-js-dev/modern.js/pull/398/files#diff-f77c749e403fbf1fb676d5687bef3d7138a230331c849298d8afaff9b6afbc3dR281
            }
          };
        },
        modifyEntryImports: function modifyEntryImports(_ref2) {
          var entrypoint = _ref2.entrypoint,
              imports = _ref2.imports;
          var userConfig = api.useResolvedConfigContext();
          var designSystem = userConfig.source.designSystem;

          if (_typeof(designSystem) === 'object' && designSystem.supportStyledComponents) {
            var designTokens = getDesignTokens(userConfig);
            imports.push({
              value: '@modern-js/runtime/plugins',
              specifiers: [{
                imported: PLUGIN_IDENTIFIER
              }],
              initialize: "\n  const designTokens = ".concat(JSON.stringify(designTokens), ";\n            ")
            });
          }

          return {
            entrypoint: entrypoint,
            imports: imports
          };
        },
        modifyEntryRuntimePlugins: function modifyEntryRuntimePlugins(_ref3) {
          var entrypoint = _ref3.entrypoint,
              plugins = _ref3.plugins;

          var _ref4 = api.useResolvedConfigContext(),
              designSystem = _ref4.source.designSystem;

          var useSCThemeProvider = true;

          if (designSystem) {
            // when designSystem exist, designToken.styledComponents`s default value is false.
            useSCThemeProvider = (designSystem === null || designSystem === void 0 ? void 0 : designSystem.supportStyledComponents) || false;
          }

          if (_typeof(designSystem) === 'object' && designSystem.supportStyledComponents) {
            plugins.push({
              name: PLUGIN_IDENTIFIER,
              options: "{token: designTokens, useStyledComponentsThemeProvider: ".concat(useSCThemeProvider ? 'true' : 'false', ", useDesignTokenContext: false}")
            });
          }

          return {
            entrypoint: entrypoint,
            plugins: plugins
          };
        },
        validateSchema: function validateSchema() {
          // add source.designSystem.supportStyledComponents config
          return PLUGIN_SCHEMAS['@modern-js/plugin-design-token'];
        },
        addRuntimeExports: function addRuntimeExports() {
          pluginsExportsUtils.addExport("export { default as designToken } from '".concat(designTokenModulePath, "'"));
        }
      };
    }
  };
});